#!/bin/bash

# URL to annotree databases are passed as commandline arguments

# Prerequisites:
# mysql setup and running
# sqlite3 setup and running
# mysql user credentials available and all privileges granted to the user

if [ $# -ne 2 ];
then
 echo "Please provide URL to latest Annotree bacteria and archea versions like: bash update_annotree_db.sh https://url.to/latest/bacteria_db https://url.to/latest/archea_db"
 exit 1
fi

# get MySQL login credentials
read -p "MySQL username: " mysqlUsername
read -sp "MySQL password: " mysqlPassword
echo

# check credentials for correctness
while ! mysql -u $mysqlUsername -p$mysqlPassword -e"quit"
do
echo "MySQL Username or Password incorrect. Please try again."
read -p "MySQL username: " mysqlUsername
read -sp "MySQL password: " mysqlPassword
echo
done

# create tmp directory
echo [$(date +"%T")] Creating 'annotree_tmp/' directory
mkdir annotree_tmp
echo

# Download MEGAN specific gtdb_id -> custom_integer mapping file (URL might be subject to change with new releases)
echo [$(date +"%T")] Downloading 'gtdb.map'
wget -O annotree_tmp/gtdb.map https://raw.githubusercontent.com/husonlab/megan-ce/master/src/megan/resources/files/gtdb.map
echo

# Loop through both database URLs
for var in "$@"
do
	# Download annotree database
	echo [$(date +"%T")] Downloading database to ./annotree_tmp/
	wget -O annotree_tmp/annotree.sql.tar.gz "$var"
	echo

	# Extract archive
	echo [$(date +"%T")] Unpacking download
	tar -xzf annotree_tmp/annotree.sql.tar.gz -C annotree_tmp/
	rm annotree_tmp/annotree.sql.tar.gz
	echo

	# Import mysql dump
	echo [$(date +"%T")] Importing annotree database into mysql
	mysql -u $mysqlUsername -p$mysqlPassword < annotree_tmp/*.sql
	echo

	# get database name from dump
	name=$(grep "CREATE DATABASE" annotree_tmp/*.sql -m 1 | grep -o '\bgtdb\w*')
	echo [$(date +"%T")] "Name of annotree database in MySQL: $name"
	rm annotree_tmp/*.sql
	echo

	# Extract gene_id, gtdb_id and protein seq from protein_sequences table and create .faa file as protein db for DIAMOND
	# The headers are a combination of the gene_id and the gtdb_id per sequence, concatenated by a double underscore
	# Dots are replaced by underscores to prevent MEGAN issues
	# Also creates accession to gtdb_id mapping file as 'annotree_tmp/accession2gtdb_id.tsv'
	echo [$(date +"%T")] Building annotree_tmp/prot_seqs_${name}.faa
	mysql -u $mysqlUsername -p$mysqlPassword -N -e "SELECT * FROM ${name}.protein_sequences;" | python3 scripts/prot_seqs_to_faa.py $name
	echo

	# Extract tables kegg_top_hits, pfam_top_hits and tigrfam_top_hits from mysql database
	echo [$(date +"%T")] Extracting table 'kegg_top_hits' from $name to annotree_tmp/
	mysql -u $mysqlUsername -p$mysqlPassword -N -e "SELECT gene_id, gtdb_id, kegg_id FROM ${name}.kegg_top_hits;" > annotree_tmp/kegg_top_hits.tsv
	echo

	echo [$(date +"%T")] Extracting table 'pfam_top_hits' from $name to annotree_tmp/
	mysql -u $mysqlUsername -p$mysqlPassword -N -e "SELECT gene_id, gtdb_id, pfam_id FROM ${name}.pfam_top_hits;" > annotree_tmp/pfam_top_hits.tsv
	echo

        echo [$(date +"%T")] Extracting table 'tigrfam_top_hits' from $name to annotree_tmp/
	mysql -u $mysqlUsername -p$mysqlPassword -N -e "SELECT gene_id, gtdb_id, tigrfam_id FROM ${name}.tigrfam_top_hits;" > annotree_tmp/tigrfam_top_hits.tsv
	echo

	echo [$(date +"%T")] Extracting table 'node_tax' from $name to annotree_tmp/
	mysql -u $mysqlUsername -p$mysqlPassword -N -e "SELECT gtdb_id, ncbi_taxid FROM ${name}.node_tax;" > annotree_tmp/node_tax.tsv
	mysql -u $mysqlUsername -p$mysqlPassword -e "DROP DATABASE ${name};"
	echo

	# Run script that builds the mapping file needed to assign functional annotations to read -> protein mappings
	echo [$(date +"%T")] Building mapping file annotree_tmp/mapping_${name}.tsv
	python3 scripts/create_mapping_file.py ${name}
	rm annotree_tmp/node_tax.tsv
	rm annotree_tmp/kegg_top_hits.tsv
	rm annotree_tmp/pfam_top_hits.tsv
	rm annotree_tmp/tigrfam_top_hits.tsv
	rm annotree_tmp/accession2gtdb_id.tsv
	echo
done

rm annotree_tmp/gtdb.map

# Join bacteria and archaea prot_seqs and mapping files
echo [$(date +"%T")] Joining Bacteria and Archaea databases
cat annotree_tmp/prot_seqs_* > annotree_prot_seqs.faa
rm annotree_tmp/prot_seqs_*

cat annotree_tmp/mapping_* > annotree_tmp/mapping.tsv
rm annotree_tmp/mapping_*
echo

# Build sqlite3 functional mapping database 
echo [$(date +"%T")] Creating sqlite3 database "annotree.db"
echo [$(date +"%T")] Importing table 'info' from scripts/sqlite_info.sql
cat scripts/sqlite_info.sql | sqlite3 annotree.db

echo [$(date +"%T")] Creating and filling table "mappings" 
sqlite3 annotree.db <<'END_SQL'
CREATE TABLE mappings (Accession PRIMARY KEY , Taxonomy INT, GTDB INT, EGGNOG INT, INTERPRO2GO INT, SEED INT, EC INT, KEGG INTEGER, PFAM INTEGER, TIGRFAM INTEGER) WITHOUT ROWID;
.separator \t
.import annotree_tmp/mapping.tsv mappings
END_SQL

rm annotree_tmp/mapping.tsv
rm -r annotree_tmp
echo

echo [$(date +"%T")] Finished.

